/* Copyright 1996, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

/* NOTE: The portion of this file below the line is derived from NCSA source,
 * and is not covered by the Backtalk copyright or license.
 */

#include "backtalk.h"
#include "str.h"
#include "cgi_query.h"
#include "dict.h"

char x2c(char *what);
void unescape_url(char *url);
void plustospace(char *str);

/* LOW-LEVEL CGI-INTERFACE
 *
 * This module contains code to decode cgi data for either GET or POST
 * methods.  It is a combination of code Jan Wolter derived from the stuff in
 * the ncsa-defaults.tar.Z distribution for web-newuser and Webspan 0.2 code
 * by Steve Weiss.  It is slightly slower than the ncsa code, but much more
 * secure.
 */


/* GET_QUERY - fetch the raw query string and return it.  It is taken either
 * from the QUERY_STRING or from stdin, depending on weather the
 * REQUEST_METHOD is POST or GET.  Sometimes the returned query string is in
 * malloc'ed memory, sometimes it isn't.  Rather a defect, if you ask me.
 */

char *get_query()
{
    char *query;
    char *method= getenv("REQUEST_METHOD");

    if (method != NULL && !strcmp(method,"POST"))
    {
	char *cl= getenv("CONTENT_LENGTH");
	int len= (cl == NULL) ? 0 : atol(cl);
	query= (char *)malloc(len+1);
	len= fread(query,1,len,stdin);
	query[len]= '\0';
    }
    else
	query= getenv("QUERY_STRING");

    return query;
}


/* LOAD_QUERY - given a raw query string, decode it and store each pair in
 * the symbol table.  No longer mangles the query.
 */

void load_query(char *query)
{
    char *p, *end, *label;

    if  (query == NULL || query[0] == '\0')
	return;
    
    p= query;
    for (;;)
    {
	label= p;

	if ((end= strchr(p,'&')) != NULL) *end= '\0';

	plustospace(p);
	unescape_url(p);

	if ((p= strchr(p,'=')) == NULL)
	    set_sys_dict(label, "", SM_KEEP);
	else
	{
	    *p= '\0';
	    set_sys_dict(label, p+1, SM_COPY);
	    *p= '=';
	}
	if (end == NULL) break;
	*end= '&';
	p= end + 1;
    }

    return;
}


/* ------- The rest of this is recycled as is from the NCSA stuff ------- */

char x2c(char *what) {
    register char digit;

    digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A')+10 : (what[0] - '0'));
    digit *= 16;
    digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A')+10 : (what[1] - '0'));
    return digit;
}

void unescape_url(char *url) {
    register int x,y;

    for(x=0,y=0;url[y];++x,++y) {
        if((url[x] = url[y]) == '%') {
            url[x] = x2c(&url[y+1]);
            y+=2;
        }
    }
    url[x] = '\0';
}

void plustospace(char *str) {
    register int x;

    for(x=0;str[x];x++) if(str[x] == '+') str[x] = ' ';
}
